using System.IO;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using FitConnect.Encryption;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using Microsoft.IdentityModel.Tokens;
using NUnit.Framework;

namespace SenderTest;

public class RootCertificatesTests {
    private ILogger? _logger;

    [OneTimeSetUp]
    public void OneTimeSetup() {
        _logger = LoggerFactory.Create(
            b => {
                b.AddSimpleConsole();
                b.SetMinimumLevel(LogLevel.Information);
            }).CreateLogger("EncryptionTests");
    }

    [Test]
    public void LoadRootCertificates_ShouldPass() {
        // Act
        var certificates = CertificateHelper.GetRootCertificates().Result;

        // Assert
        certificates.Count.Should().BeGreaterThan(2);
    }

    [Test]
    public void ProdSubmission_ValidCertificate_ShouldPass() {
        // Arrange
        var certificate = File.ReadAllText("./Certificates/ValidCertificateFromProdPki.json");
        var keySetToBeTested = new JsonWebKeySet(certificate).Keys.First();

        // Act
        var result = new CertificateHelper(_logger)
            .ValidateCertificateJsonWebKeyAsync(keySetToBeTested, null, LogLevel.Trace)
            .Result;

        // Assert
        result.Should().BeTrue();
    }

    [Test]
    public void TestPki_ValidCertificate_ShouldPass() {
        // Arrange
        var root =
            X509Certificate2.CreateFromPem(
                File.ReadAllText("./Certificates/TestPkiRootCertificate.pem"));

        var keySetToBeTested =
            new JsonWebKey(
                File.ReadAllText("./Certificates/ValidCertificateFromTestPki.json"));

        // Act
        var result = new CertificateHelper(_logger)
            .ValidateCertificateJsonWebKeyAsync(keySetToBeTested, new[] { root }, LogLevel.Trace)
            .Result;

        // Assert
        result.Should().BeTrue();
    }

    [Test]
    public void TestPki_RevokedCertificate_ShouldFail() {
        // Arrange
        var root =
            X509Certificate2.CreateFromPem(
                File.ReadAllText("./Certificates/TestPkiRootCertificate.pem"));

        var keySetToBeTested =
            new JsonWebKey(
                File.ReadAllText("./Certificates/RevokedCertificate.json"));

        // Act
        var result =
            new CertificateHelper(_logger).ValidateCertificateJsonWebKeyAsync(keySetToBeTested,
                new[] { root },
                LogLevel.Trace).Result;

        // Assert
        result.Should().BeFalse();
    }

    [Test]
    public void TestPkiCertificate_ValidatesAgainstProdPki_ShouldFail() {
        // Arrange
        var keySetToBeTested =
            new JsonWebKey(
                File.ReadAllText("./Certificates/RevokedCertificate.json"));

        // Act
        var result =
            new CertificateHelper(_logger).ValidateCertificateJsonWebKeyAsync(keySetToBeTested,
                logLevel: LogLevel.Trace).Result;

        // Assert
        result.Should().BeFalse();
    }

    [Test]
    public void TestKeySetAsKey_ShouldFail() {
        // Arrange
        var root =
            X509Certificate2.CreateFromPem(
                File.ReadAllText("./Certificates/TestPkiRootCertificate.pem"));

        var keySetToBeTested =
            new JsonWebKeySet(
                File.ReadAllText("./Certificates/ValidCertificateFromTestPki.json"));

        // Act
        var result =
            new CertificateHelper(_logger).ValidateCertificateJsonWebKeySet(keySetToBeTested,
                new[] { root },
                LogLevel.Trace).Result;

        // Assert
        result.Should().BeFalse();
    }

    [Test]
    public void RevokedKeyAsKeySet_ShouldFail() {
        // Arrange
        var root =
            X509Certificate2.CreateFromPem(
                File.ReadAllText("./Certificates/TestPkiRootCertificate.pem"));

        var keySetToBeTested =
            new JsonWebKeySet(
                File.ReadAllText("./Certificates/RevokedCertificate.json"));

        // Act
        var result =
            new CertificateHelper(_logger).ValidateCertificateJsonWebKeySet(keySetToBeTested,
                new[] { root },
                LogLevel.Trace).Result;

        // Assert
        result.Should().BeFalse();
    }
}
